#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@

from __future__ import division

"""
Modules to provide some 2.7 features in 2.6.
"""

NO_VALUE = object()


class MyOrderedDict(dict):
    @classmethod
    def fromkeys(cls, seq, value=None):
        return cls((s, value) for s in seq)

    def __init__(self, items=None):
        if items is None:
            items = []
        elif isinstance(items, dict):
            items = [(k, v) for k, v in items.items()]
        else:
            # Unwind possible iterators
            items = list(items)
        self._keys = [i[0] for i in items]
        self._dict = dict(items)
        for k, v in items:
            self[k] = v
    def __eq__(self, other):
        return (self._keys == other._keys
                and self._dict == other._dict)
    def __len__(self):
        return len(self._keys)
    def __getitem__(self, k):
        return self._dict[k]
    def __setitem__(self, k, v):
        if k not in self._keys:
            self._keys.append(k)
        self._dict[k] = v
    def __delitem__(self, k):
        try:
            idx = self._keys.index(k)
        except ValueError:
            return
        newkeys = self._keys[:idx] + self._keys[idx + 1:]
        newdict = dict(self._dict)
        newdict.pop(k)
        self._keys = newkeys
        self._dict = newdict
    def __contains__(self, k):
        return k in self._keys
    def items(self):
        return [(k, self[k]) for k in self._keys]
    def keys(self):
        return [k for k in self._keys]
    def values(self):
        return [self[k] for k in self._keys]
    def iteritems(self):
        return iter(self.items())
    def iterkeys(self):
        return iter(self.keys())
    def itervalues(self):
        return iter(self.values())
    def __iter__(self):
        return self.iterkeys()
    def __reversed__(self):
        for k in reversed(self._keys):
            yield k
    def has_key(self, k):
        return k in self._keys
    def get(self, k, default=None):
        try:
            return self[k]
        except KeyError:
            return default
    def setdefault(self, k, default=None):
        try:
            return self[k]
        except KeyError:
            self[k] = default
            return default
    def clear(self):
        self._keys = []
        self._dict = {}
    def copy(self):
        return OrderedDict(self.items())
    def pop(self, k, default=NO_VALUE):
        try:
            rc = self[k]
        except KeyError:
            if default is not NO_VALUE:
                return default
            else:
                raise
        del(k)
        return rc
    def popitem(self, last=True):
        if len(self) == 0: raise KeyError("Dictionary is empty")
        if last:
            rc = self._keys[-1]
        else:
            rc = self._keys[0]
        return rc, self.pop(rc)
    def update(self, other=NO_VALUE):
        if other is NO_VALUE:
            return
        if isinstance(other, dict):
            other_iter = other.iteritems()
        else:
            other_iter = iter(other)
        for k, v in other_iter:
            self[k] = v
    def __repr__(self):
        return "OrderedDict(" + str(self.items()) + ")"

    # Not supporting: viewitems, viewkeys, viewvalues
    def viewitems(self): raise NotImplementedError("viewitems")
    def viewkeys(self): raise NotImplementedError("viewkeys")
    def viewvalues(self): raise NotImplementedError("viewvalues")
    


try:
    from collections import OrderedDict
except ImportError:
    OrderedDict = MyOrderedDict
