#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
from __future__ import division
import sys
import cairo
import backend_cairo

import samplecharts


def main():
    command = sys.argv[1]
    outfile = sys.argv[2]
    width = int(sys.argv[3])
    height = int(sys.argv[4])
    
    if outfile.endswith(".svg"):
        surface = cairo.SVGSurface(outfile, width, height)
    elif outfile.endswith(".png"):
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32,
                                     width, height)
    else:
        raise Exception("Unknown image format for '%s'" % outfile)
    
    cr = cairo.Context(surface)

    # Draw a dummy background if this is a PNG
    if outfile.endswith(".png"):
        cr.set_source_rgb(1, 1, 1);
        cr.paint()
    

    ctx = backend_cairo.Context(cr, width, height)


    chart_functions = {
        "dots" : lambda : samplecharts.dots_and_lines(ctx,
                                                      sys.argv[5],
                                                      sys.argv[6]),
        "ts"   : lambda : samplecharts.timeseries(ctx, sys.argv[5]),
        "exist": lambda : samplecharts.existence(ctx, sys.argv[5],
                                                 lo=100,
                                                 med=1000,
                                                 hi=10000),
        "clouds": lambda : samplecharts.cloud_chamber(ctx, sys.argv[5])
    }

    chart = chart_functions[command]()
    chart(ctx)

    if outfile.endswith(".png"):
        surface.write_to_png(outfile)




main()
