#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@

from __future__ import division
import unittest

# backgrounds has an old copy of Bitmap in it; I am importing the
# newer copy of Bitmap into the test namespace, but we will test with
# old Bitmap as well, so, import rayon.backgrounds, too.
import rayon.backgrounds
from rayon.backgrounds import (
    BitmapBackground,
    ColorBackground,
    CompositeBackground,
    PNGBackground
)

class FakeContext(object):
    def __init__(self):
        self._calls = []

    def test_get_calls(self): return self._calls

    def draw_rectangle(self,
                       x,
                       y,
                       width,
                       height,
                       stroke_width,
                       stroke_color,
                       fill_color,
                       fixup=False):
        self._calls.append(
            ['draw_rectangle',
             { 'x'            : x,
               'y'            : y,
               'width'        : width,
               'height'       : height,
               'stroke_width' : stroke_width,
               'stroke_color' : stroke_color,
               'fill_color'   : fill_color,
               'fixup'        : fixup}])

    def draw_bitmap(self, bitmap, x, y, width, height, fill):
        self._calls.append(
            ['draw_bitmap',
             { 'x'            : x,
               'y'            : y,
               'width'        : width,
               'height'       : height,
               'fill'         : fill}])

    def draw_image(self, filename, x, y, width, height, fill):
        self._calls.append(
            ['draw_image',
             { 'filename'     : filename,
               'x'            : x,
               'y'            : y,
               'width'        : width,
               'height'       : height,
               'fill'         : fill}])
        

class FakeBackground(object):
    def __init__(self):
        self._calls = []

    def test_get_calls(self): return self._calls

    def draw(self, ctx, width, height):
        self._calls.append(
            ['draw',
             {'ctx'    : ctx,
              'width'  : width,
              'height' : height}])
        
    

class t_Background(unittest.TestCase):

    def test_color_background_1(self):
        fill_color=(255, 0, 0, 255)
        stroke_color=(0, 255, 0, 255)
        ctx = FakeContext()
        bg = ColorBackground(fill_color=fill_color, stroke_color=stroke_color)
        bg.draw(ctx, 10, 20)
        self.assertEqual(ctx.test_get_calls()[0],
                         ['draw_rectangle',
                          { 'x'            : 0,
                            'y'            : 20,
                            'width'        : 10,
                            'height'       : 20,
                            'stroke_width' : 1,
                            'stroke_color' : stroke_color,
                            'fill_color'   : fill_color,
                           'fixup'        : False}])

    def test_bitmap_background_1(self):
        ctx = FakeContext()
        fake_bitmap = object()
        bg = BitmapBackground(fake_bitmap, fill="none")
        bg.draw(ctx, 10, 20)
        self.assertEqual(ctx.test_get_calls()[0],
            ['draw_bitmap',
             { 'x'            : 0,
               'y'            : 0,
               'width'        : 10,
               'height'       : 20,
               'fill'         : "none"}])


    def test_png_background_1(self):
        fname = "filename.png"
        ctx = FakeContext()
        bg = PNGBackground(fname, fill="none")
        bg.draw(ctx, 10, 20)
        self.assertEqual(ctx.test_get_calls()[0],
            ['draw_image',
             { 'filename'     : fname,
               'x'            : 0,
               'y'            : 0,
               'width'        : 10,
               'height'       : 20,
               'fill'         : "none"}])
        
    def test_composite_background_1(self):
        ctx = object()
        bg1 = FakeBackground()
        bg2 = FakeBackground()
        bg = CompositeBackground()
        bg.add(bg1)
        bg.add(bg2)
        bg.draw(ctx, 10, 20)
        # TODO: test that these are called in order
        self.assertEqual(bg1.test_get_calls()[0],
                         ['draw',
                          {'ctx': ctx,
                           'width': 10,
                           'height': 20}])
        self.assertEqual(bg2.test_get_calls()[0],
                         ['draw',
                          {'ctx': ctx,
                           'width': 10,
                           'height': 20}])

if __name__ == '__main__':
    unittest.main()
