#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@

from __future__ import division

import unittest
from math import pi


from rayon.geometry import (CartesianPoint,
                            ScreenPoint,
                            PolarPoint,
                            cartesian_from_polar,
                            polar_from_cartesian,
                            screen_from_cartesian,
                            cartesian_from_screen)

class t_CartesianPoint(unittest.TestCase):
    def test_access_1(self):
        c = CartesianPoint(10, 20)
        x, y = c
        self.assertEqual(x, 10)
        self.assertEqual(y, 20)
    def test_access_2(self):
        c = CartesianPoint(10, 20)
        self.assertEqual(c[0], 10)
        self.assertEqual(c[1], 20)
    def test_access_err_1(self):
        c = CartesianPoint(10, 20)
        self.assertRaises(IndexError, lambda: c[3])
    def test_eq_1(self):
        self.assertEqual(CartesianPoint(10, 20), CartesianPoint(10, 20))
    def test_eq_2(self):
        self.assertEqual(CartesianPoint(10, 20), CartesianPoint(10.0, 20))
    def test_eq_3(self):
        self.assertEqual(CartesianPoint(10, 20), CartesianPoint(10, 20.0))
    def test_neq_1(self):
        self.assertNotEqual(CartesianPoint(10, 20), CartesianPoint(20, 20))
    def test_neq_2(self):
        self.assertNotEqual(CartesianPoint(10, 20), ScreenPoint(10, 20))
    def test_neq_3(self):
        self.assertNotEqual(CartesianPoint(10, 20), PolarPoint(1, .5*pi))
    def test_neq_4(self):
        self.assertNotEqual(CartesianPoint(10, 20), (10, 20))
    def test_add_1(self):
        self.assertEqual(CartesianPoint(10, 20) + CartesianPoint(5, 6),
                         CartesianPoint(15, 26))
    def test_sub_1(self):
        self.assertEqual(CartesianPoint(15, 26) - CartesianPoint(5, 6),
                         CartesianPoint(10, 20))
    def test_sub_2(self):
        self.assertEqual(CartesianPoint(5, 6) - CartesianPoint(10, 20),
                         CartesianPoint(-5, -14))

        
class t_ScreenPoint(unittest.TestCase):
    def test_access_1(self):
        s = ScreenPoint(10, 20)
        x, y = s
        self.assertEqual(x, 10)
        self.assertEqual(y, 20)
    def test_access_2(self):
        s = ScreenPoint(10, 20)
        self.assertEqual(s[0], 10)
        self.assertEqual(s[1], 20)
    def test_access_err_1(self):
        s = ScreenPoint(10, 20)
        self.assertRaises(IndexError, lambda: s[3])
    def test_eq_1(self):
        self.assertEqual(ScreenPoint(10, 20), ScreenPoint(10, 20))
    def test_eq_2(self):
        self.assertEqual(ScreenPoint(10, 20), ScreenPoint(10.0, 20))
    def test_eq_3(self):
        self.assertEqual(ScreenPoint(10, 20), ScreenPoint(10, 20.0))
    def test_neq_1(self):
        self.assertNotEqual(ScreenPoint(10, 20), ScreenPoint(20, 10))
    def test_neq_2(self):
        self.assertNotEqual(ScreenPoint(10, 20), CartesianPoint(10, 20))
    def test_neq_3(self):
        self.assertNotEqual(ScreenPoint(10, 20), PolarPoint(1, .5*pi))
    def test_neq_4(self):
        self.assertNotEqual(ScreenPoint(10, 20), (10, 20))
    def test_add_1(self):
        self.assertEqual(ScreenPoint(10, 20) + ScreenPoint(5, 6),
                         ScreenPoint(15, 26))
    def test_sub_1(self):
        self.assertEqual(ScreenPoint(15, 26) - ScreenPoint(5, 6),
                         ScreenPoint(10, 20))
    def test_sub_2(self):
        self.assertEqual(ScreenPoint(5, 6) - ScreenPoint(10, 20),
                         ScreenPoint(-5, -14))
    


        
class t_PolarPoint(unittest.TestCase):
    def test_access_1(self):
        p = PolarPoint(10, .5*pi)
        x, y = p
        self.assertEqual(x, 10)
        self.assertEqual(y, .5*pi)
    def test_access_2(self):
        p = PolarPoint(10, .5*pi)
        self.assertEqual(p[0], 10)
        self.assertEqual(p[1], .5*pi)
    def test_access_err_1(self):
        p = PolarPoint(10, .5*pi)
        self.assertRaises(IndexError, lambda: p[3])
    def test_eq_1(self):
        self.assertEqual(PolarPoint(10, .5*pi), PolarPoint(10, .5*pi))
    def test_eq_2(self):
        self.assertEqual(PolarPoint(10, .5*pi), PolarPoint(10.0, .5*pi))
    def test_eq_3(self):
        self.assertEqual(PolarPoint(10, 0), PolarPoint(10, 0.0))
    def test_neq_1(self):
        self.assertNotEqual(PolarPoint(10, .5*pi), PolarPoint(0, .5*pi))
    def test_neq_2(self):
        self.assertNotEqual(PolarPoint(10, .5*pi), CartesianPoint(10, 20))
    def test_neq_3(self):
        self.assertNotEqual(PolarPoint(10, .5*pi), ScreenPoint(10, 20))
    def test_neq_4(self):
        self.assertNotEqual(PolarPoint(10, .5*pi), (10, .5*pi))

class t_cartesian_from_polar(unittest.TestCase):
    def test_1(self):
        c = cartesian_from_polar(PolarPoint(10, .3*pi))
        self.assert_(isinstance(c, CartesianPoint))
    def test_2(self):
        x, y = cartesian_from_polar(PolarPoint(10, .3*pi))
        self.assertAlmostEqual(x, 5.87785, places=5)
        self.assertAlmostEqual(y, 8.09017, places=5)
    def test_err_1(self):
        self.assertRaises(
            TypeError,
            lambda: cartesian_from_polar((10, .3*pi)))

class t_polar_from_cartesian(unittest.TestCase):
    def test_1(self):
        p = polar_from_cartesian(CartesianPoint(0, 5))
        self.assert_(isinstance(p, PolarPoint))
    def test_2(self):
        r, theta = polar_from_cartesian(CartesianPoint(0, 5))
        self.assertAlmostEqual(r, 5)
        self.assertAlmostEqual(theta, .5*pi)
    def test_err_1(self):
        self.assertRaises(
            TypeError,
            lambda: polar_from_cartesian((0, 5)))


class t_screen_from_cartesian(unittest.TestCase):
    def test_1(self):
        s = screen_from_cartesian(CartesianPoint(10, 20), 100)
        self.assert_(isinstance(s, ScreenPoint))
    def test_2(self):
        x, y = screen_from_cartesian(CartesianPoint(10, 20), 100)
        self.assertEqual(x, 10)
        self.assertEqual(y, 80)
    def test_3(self):
        x, y = screen_from_cartesian(CartesianPoint(.25, .25), 1)
        self.assertAlmostEqual(x, .25)
        self.assertAlmostEqual(y, .75)
    def test_err_1(self):
        self.assertRaises(
            TypeError,
            lambda: screen_from_cartesian((.25, .25), 1))

class t_cartesian_from_screen(unittest.TestCase):
    def test_1(self):
        c = cartesian_from_screen(ScreenPoint(10, 20), 100)
        self.assert_(isinstance(c, CartesianPoint))
    def test_2(self):
        x, y = cartesian_from_screen(ScreenPoint(10, 80), 100)
        self.assertEqual(x, 10)
        self.assertEqual(y, 20)
    def test_3(self):
        x, y = cartesian_from_screen(ScreenPoint(.25, .75), 1)
        self.assertAlmostEqual(x, .25)
        self.assertAlmostEqual(y, .25)
    def test_err_1(self):
        self.assertRaises(
            TypeError,
            lambda: cartesian_from_screen((.25, .25), 1))
        

if __name__ == '__main__':
    unittest.main()
