import unittest

import datetime

from rayon.miscutils import *
#from rayon.datatypes import IPv4
from netsa.data.times import make_datetime
from netsa_silk import IPAddr

def is_ipv6(addr):
    try:
        return len(addr.octets()) == 16
    except:
        return False

def is_ipv4(addr):
    try:
        rc = len(addr.octets())
        print "rc == %d" % rc
        return len(addr.octets()) == 4
    except:
        raise RuntimeError("contents of addr: %s\ntype of addr: %s" % (dir(addr), type(addr)))
        return False



class t_infer_type_from_string(unittest.TestCase):
    def test_str_1(self):
        self.assertEquals(str, infer_type_from_string("phil"))
    def test_int_1(self):
        self.assertEquals(int, infer_type_from_string("123"))
    def test_float_1(self):
        self.assertEquals(float, infer_type_from_string("12.3"))
    def test_datetime_1(self):
        self.assertEquals(make_datetime, 
                          infer_type_from_string("2001/01/01:01:02:03"))
    # def test_ipv4_1(self):
    #     self.assertEquals(IPv4, infer_type_from_string("1.2.3.4"))
    def test_ipv4_2(self):
        self.assertEquals(str, infer_type_from_string("1.2.34"))
    def test_ipv4_3(self):
        self.assertEquals(netsa_silk.IPAddr, infer_type_from_string("0.2.3.4"))
    def test_ipv4_4(self):
        self.assertEquals(str, infer_type_from_string("1.2.256.4"))
    def test_ipv4_5(self):
        self.assertEquals(str, infer_type_from_string("1.2.256.z"))
        
    def test_ipv4_netmask_1(self):
        # not currently supported
        self.assertEquals(str, infer_type_from_string("1.2.3.4/32"))
    def test_ipv4_netmask_2(self):
        # not currently supported
        self.assertEquals(str, infer_type_from_string("1.2.3.0/24"))


    def test_ipv6_1(self):
        self.assertEquals(IPAddr, infer_type_from_string("::"))
    def test_ipv6_2(self):
        self.assertEquals(IPAddr, infer_type_from_string("::1"))
    def test_ipv6_3(self):
        self.assertEquals(IPAddr, infer_type_from_string("::1.2.3.4"))
    def test_ipv6_4(self):
        self.assertEquals(IPAddr, infer_type_from_string(
                    "0:0:0:0:0:0:1.2.3.4"))
    def test_ipv6_5(self):
        self.assertEquals(IPAddr, infer_type_from_string(
                    "0000:0000:0000:0000:0000:0000:1.2.3.4"))
    def test_ipv6_6(self):
        self.assertEquals(
            str, 
            infer_type_from_string("0000:0000:0000:0000:0000:0000:300.2.3.4"))
    def test_ipv6_7(self):
        self.assertEquals(IPAddr, infer_type_from_string(
                    "0123:1234:2345:3456:4567:5678:6789:7890"))
    def test_ipv6_8(self):
        self.assertEquals(IPAddr, infer_type_from_string(
                    "dead:beef:dead:beef:dead:beef:dead:beef"))
    def test_ipv6_9(self):
        self.assertEquals(
            str, 
            infer_type_from_string("degd:beef:degd:beef:degd:beef:degd:beef"))
