from rayon.raster import *

import unittest
import sys
import array


def t_get_data(pixel, expected, in_fmt=RGBA32, out_fmt=RGBA32, pdb_=False):
    def run_test(self):
        bm = Bitmap(1, 1, fmt=in_fmt)
        bm.set_pixel(0, pixel)
        self.assertEqual(bm.get_data(fmt=out_fmt), array.array("B", expected))
    return run_test

        
class t_Bitmap(unittest.TestCase):
    def rgba(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=RGBA32, out_fmt=RGBA32)
    test_rgba_1 = rgba((255, 0, 0, 0), (255, 0, 0, 0))
    test_rgba_2 = rgba((0, 255, 0, 0), (0, 255, 0, 0))
    test_rgba_3 = rgba((0, 0, 255, 0), (0, 0, 255, 0))
    test_rgba_4 = rgba((0, 0, 0, 255), (0, 0, 0, 255))

    
    def rgba_pm_in(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=RGBA32_PM, out_fmt=RGBA32)
    test_rgba_pm_in_1 = rgba_pm_in((128, 0, 0, 128), (255, 0, 0, 128))
    test_rgba_pm_in_2 = rgba_pm_in((0, 128, 0, 128), (0, 255, 0, 128))
    test_rgba_pm_in_3 = rgba_pm_in((0, 0, 128, 128), (0, 0, 255, 128))

    def rgba_pm_out(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=RGBA32, out_fmt=RGBA32_PM)
    test_rgba_pm_out_1 = rgba_pm_out((255, 0, 0, 128), (128, 0, 0, 128))
    test_rgba_pm_out_2 = rgba_pm_out((0, 255, 0, 128), (0, 128, 0, 128))
    test_rgba_pm_out_3 = rgba_pm_out((0, 0, 255, 128), (0, 0, 128, 128))

    
    def argb_in(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=ARGB32, out_fmt=RGBA32)
    test_argb_in_1 = argb_in((0, 255, 0, 0), (255, 0, 0, 0))
    test_argb_in_1 = argb_in((0, 0, 255, 0), (0, 255, 0, 0))
    test_argb_in_1 = argb_in((0, 0, 0, 255), (0, 0, 255, 0))
    test_argb_in_1 = argb_in((255, 0, 0, 0), (0, 0, 0, 255))

    def argb_out(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=RGBA32, out_fmt=ARGB32)
    test_argb_out_1 = argb_out((255, 0, 0, 0), (0, 255, 0, 0))
    test_argb_out_2 = argb_out((0, 255, 0, 0), (0, 0, 255, 0))
    test_argb_out_3 = argb_out((0, 0, 255, 0), (0, 0, 0, 255))
    test_argb_out_4 = argb_out((0, 0, 0, 255), (255, 0, 0, 0))

    
    def argb_pm_in(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=ARGB32_PM, out_fmt=RGBA32)
    test_argb_pm_in_1 = argb_pm_in((128, 128, 0, 0), (255, 0, 0, 128))
    test_argb_pm_in_2 = argb_pm_in((128, 0, 128, 0), (0, 255, 0, 128))
    test_argb_pm_in_3 = argb_pm_in((128, 0, 0, 128), (0, 0, 255, 128))
        
    def argb_pm_out(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=RGBA32, out_fmt=ARGB32_PM)
    test_argb_pm_1 = argb_pm_out((255, 0, 0, 128), (128, 128, 0, 0))
    test_argb_pm_2 = argb_pm_out((0, 255, 0, 128), (128, 0, 128, 0))
    test_argb_pm_3 = argb_pm_out((0, 0, 255, 128), (128, 0, 0, 128))


    
    def bgra_in(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=BGRA32, out_fmt=RGBA32)
    test_bgra_in_1 = bgra_in((0, 0, 255, 0), (255, 0, 0, 0))
    test_bgra_in_1 = bgra_in((0, 255, 0, 0), (0, 255, 0, 0))
    test_bgra_in_1 = bgra_in((255, 0, 0, 0), (0, 0, 255, 0))

    def bgra_out(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=RGBA32, out_fmt=BGRA32)
    test_bgra_out_1 = bgra_out((255, 0, 0, 0), (0, 0, 255, 0))
    test_bgra_out_2 = bgra_out((0, 255, 0, 0), (0, 255, 0, 0))
    test_bgra_out_3 = bgra_out((0, 0, 255, 0), (255, 0, 0, 0))
    test_bgra_out_4 = bgra_out((0, 0, 0, 255), (0, 0, 0, 255))

    
    def bgra_pm_in(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=BGRA32_PM, out_fmt=RGBA32)
    test_bgra_pm_in_1 = bgra_pm_in((0, 0, 128, 128), (255, 0, 0, 128))
    test_bgra_pm_in_2 = bgra_pm_in((0, 128, 0, 128), (0, 255, 0, 128))
    test_bgra_pm_in_3 = bgra_pm_in((128, 0, 0, 128), (0, 0, 255, 128))
        
    def bgra_pm_out(pixel, expected):
        return t_get_data(pixel, expected, in_fmt=RGBA32, out_fmt=BGRA32_PM)
    test_bgra_pm_out_1 = bgra_pm_out((255, 0, 0, 128), (0, 0, 128, 128))
    test_bgra_pm_out_2 = bgra_pm_out((0, 255, 0, 128), (0, 128, 0, 128))
    test_bgra_pm_out_3 = bgra_pm_out((0, 0, 255, 128), (128, 0, 0, 128))


    
    

    def test_illegal_format_1(self):
        self.assertRaises(ValueError, lambda: Bitmap(1, 1, fmt="phil"))

    def test_get_pixel_rgba_1(self):
        bm = Bitmap(1, 1)
        bm.set_pixel(0, (25, 50, 100, 200))
        self.assertEqual(bm.get_pixel(0, fmt=RGBA32), (25, 50, 100, 200))

    def test_get_pixel_rgba_2(self):
        bm = Bitmap(1, 1)
        bm.set_pixel(0, (25, 50, 100, 200))
        self.assertEqual(bm.get_pixel(0, fmt=RGBA32), (25, 50, 100, 200))

    def test_get_pixel_rgba_pm(self):
        bm = Bitmap(1, 1)
        # Pixel is set in RGBA32...
        bm.set_pixel(0, (24, 50, 100, 128))
        # ...but you can get it out in RGBA32_PM
        self.assertEqual(bm.get_pixel(0, fmt=RGBA32_PM), (12, 25, 50, 128))

    def test_get_pixel_argb(self):
        bm = Bitmap(1, 1)
        bm.set_pixel(0, (25, 50, 100, 200))
        self.assertEqual(bm.get_pixel(0, fmt=ARGB32), (200, 25, 50, 100))

    def test_get_pixel_argb_pm(self):
        bm = Bitmap(1, 1)
        bm.set_pixel(0, (24, 50, 100, 128))
        self.assertEqual(bm.get_pixel(0, fmt=ARGB32_PM), (128, 12, 25, 50))


    def test_get_width(self):
        self.assertEqual(Bitmap(10, 20).get_width(), 10)
        
    def test_get_height(self):
        self.assertEqual(Bitmap(10, 20).get_height(), 20)


    def test_setitem(self):
        bm = Bitmap(1, 1)
        bm[0] = (1,2,3,4)
        self.assertEqual(bm.get_pixel(0), (1, 2, 3, 4))

    def test_getitem(self):
        bm = Bitmap(1, 1)
        bm.set_pixel(0, (1,2,3,4))
        self.assertEqual(bm[0], (1,2,3,4))

    def test_len(self):
        bm = Bitmap(1, 1)
        bm.set_pixel(0, (1, 2, 3, 4))
        self.assertEqual(len(bm), 1)

    def test_from_initializer(self):
        bm = Bitmap(1, 1, initializer=lambda idx: (1, 2, 3, 4))
        self.assertEqual(len(bm), 1)
        self.assertEqual(bm.get_pixel(0), (1,2,3,4))

    def test_from_buffer(self):
        buff = array.array('B', [1,2,3,4])
        bm = Bitmap(1, 1, buff=buff)
        self.assertEqual(bm.get_data(), buff)

    def test_upscale_1(self):
        bm = Bitmap(2, 2)
        bm.set_pixel(0, (1,2,3,4))
        bm.set_pixel(3, (2,4,6,8))
        expected = [1,2,3,4, 1,2,3,4, 0,0,0,0, 0,0,0,0,
                    1,2,3,4, 1,2,3,4, 0,0,0,0, 0,0,0,0,
                    0,0,0,0, 0,0,0,0, 2,4,6,8, 2,4,6,8,
                    0,0,0,0, 0,0,0,0, 2,4,6,8, 2,4,6,8,]
        bm2 = bm.upscale(2)
        self.assertEqual(bm2.get_data(), array.array('B', expected))
        
                         

if __name__ == '__main__':
    unittest.main()
