#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@

from __future__ import division

import unittest

from rayon.testutils import *

class t_Mock(unittest.TestCase):
    def test_return_value_1(self):
        m = Mock({"foo": 1})
        self.assertEquals(m.foo(), 1)
    def test_name_fail_1(self):
        m = Mock({"foo": 1})
        self.assertRaises(AttributeError, lambda: m.bar())
    def test_args_positional_1(self):
        m = Mock({"foo": 1})
        m.foo('a', 'b', 'c')
        m.check_call(0, "foo", 'a', 'b', 'c')
    def test_args_positional_fail_1(self):
        m = Mock({"foo": 1})
        m.foo('a', 'b', 'c')
        self.assertRaises(
            AssertionError,
            lambda: m.check_call(0, "foo", 'a', 'b', 'd'))
    def test_args_named_1(self):
        m = Mock({"foo": 1})
        m.foo(bar="bax")
        m.check_call(0, "foo", bar="bax")
    def test_args_named_fail_1(self):
        m = Mock({"foo": 1})
        m.foo(bar="bax")
        self.assertRaises(
            AssertionError,
            lambda: m.check_call(0, "foo", barber="bax"))
    def test_args_hybrid_1(self):
        m = Mock({"foo": 1})
        m.foo(1, 2, bar="bax")
        m.check_call(0, "foo", 1, 2, bar="bax")
    def test_args_hybrid_fail_1(self):
        m = Mock({"foo": 1})
        m.foo(1, 2, bar="bax")
        self.assertRaises(
            AssertionError,
            lambda: m.check_call(0, "foo", bar="bax"))
        self.assertRaises(
            AssertionError,
            lambda: m.check_call(0, "foo", 1,2))
    def test_exception_1(self):
        m = Mock({"foo": Raises(TypeError("Hindley-Milner"))})
        self.assertRaises(TypeError, m.foo)
    def test_unbound_method_1(self):
        m = Mock()
        m.foo = lambda x: x
        self.assertEqual(m.foo(4), 4)
    def test_bound_method_1(self):
        m = Mock({'foo': BoundMethod(lambda self, x: (self, x))})
        self.assertEqual(m.foo(4), (m, 4))
    def test_cycle_1(self):
        m = Mock({"foo": Cycle(0, 1)})
        vals = m.foo(), m.foo(), m.foo(), m.foo()
        self.assertEqual(vals, (0, 1, 0, 1))
    def test_cycle_2(self):
        m = Mock({"foo": Cycle(0, 1, Raises(RuntimeError), 2)})
        self.assertEqual((m.foo(), m.foo()), (0, 1))
        self.assertRaises(RuntimeError, lambda: m.foo())
        self.assertEqual((m.foo(), m.foo()), (2, 0))
    def test_empty_1(self):
        m = Mock()
        self.assertRaises(AttributeError, lambda: m.foo())
    def test_no_calls_1(self):
        m = Mock()
        self.assertRaises(AssertionError, lambda: m.check_call(0, "foo"))
            
if __name__ == '__main__':
    unittest.main()
