#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
import sys
import math
import wx

class MainFrame(wx.Frame):
    def __init__(self, title, size):
        wx.Frame.__init__(self, None, -1, title, size=size,
                          style=wx.DEFAULT_FRAME_STYLE)
        self.sz = wx.BoxSizer(wx.HORIZONTAL)
    def add_children(self, *children):
        for child in children:
            self.sz.Add(child)
        self.sz.Fit(self)

class TestPanel(wx.Panel):
    def __init__(self, parent, size):
        wx.Panel.__init__(self, parent, size=size)
        self.Bind(wx.EVT_PAINT, self.on_paint)
    def draw_text_gc(self, degs, gc, dc):
        msg = "|............RrQq1...%s" % degs
        # NOTE: Windows, at least, respects the 'face' argument to
        # wx.Font. GTK and Mac(?) need Cairo support for that to work
        font = wx.Font(10, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,
                       wx.FONTWEIGHT_NORMAL)
        print font.GetNativeFontInfoUserDesc()
        gc.SetFont(font)
        gc.PushState()
        try:
            gc.Translate(200, 200)
            gc.DrawRotatedText(msg, 0, 0, math.radians(degs))
        finally:
            gc.PopState()
    def draw_text_dc(self, degs, gc, dc):
        font = wx.Font(14, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
                       wx.FONTWEIGHT_NORMAL)
        old_font = dc.GetFont()
        dc.SetFont(font)
        try:
            msg = "|...................%s" % degs
            dc.DrawRotatedText(msg, 200, 200, degs)
        finally:
            dc.SetFont(old_font)
    def draw_lines(self, gc, dc):
        gc.SetPen(wx.Pen((0,0,0), 1))
        if wx.Platform == '__WXGTK__':
            gc.StrokeLine(100, 200.5, 300, 200.5)
            gc.StrokeLine(200.5, 100, 200.5, 300)
        else:
            gc.StrokeLine(100, 200, 300, 200)
            gc.StrokeLine(200, 100, 200, 300)
            
    def on_paint(self, evt):
        pdc = wx.PaintDC(self)
        gc = wx.GraphicsContext.Create(pdc)
        if sys.argv[1] == 'text':
            for degs in xrange(0, 360, 45):
                meth = "draw_text_%s" % sys.argv[2]
                if hasattr(self, meth):
                    meth = getattr(self, meth)
                    meth(degs, gc, pdc)
                else:
                    print "No method %s" % meth
        elif sys.argv[1] == 'lines':
            self.draw_lines(gc, pdc)
                
    
class TheApp(wx.App):
    def __init__(self):
        wx.App.__init__(self, redirect=False,
                        filename=None, useBestVisual=True)
        self.frame = MainFrame("phil", (400, 400))
        p = TestPanel(self.frame, (400, 400))
        self.frame.add_children(p)
        self.frame.Show()
        self.SetTopWindow(self.frame)
        self.Bind(wx.EVT_CLOSE, self.on_quit)
    def on_quit(self, evt):
        self.frame.Close()
        self.Exit()
           

app = TheApp()
app.MainLoop()
